@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2020 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2022 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -23,8 +23,8 @@
 #if SDL_VIDEO_DRIVER_NACL
 
 #include "SDL.h"
-#include "../../events/SDL_sysevents.h"
 #include "../../events/SDL_events_c.h"
+#include "../SDL_sysvideo.h"
 #include "SDL_naclevents_c.h"
 #include "SDL_naclvideo.h"
 #include "ppapi_simple/ps_event.h"
