@@ -17,6 +17,7 @@
 #include <string.h>
 
 #include "SDL.h"
+#include "testutils.h"
 
 #ifdef __EMSCRIPTEN__
 #include <emscripten/emscripten.h>
@@ -27,6 +28,10 @@
 #define SCREEN_WIDTH    512
 #define SCREEN_HEIGHT   320
 
+#define BUTTON_SIZE     50
+#define AXIS_SIZE       50
+
+
 /* This is indexed by SDL_GameControllerButton. */
 static const struct { int x; int y; } button_positions[] = {
     {387, 167},  /* SDL_CONTROLLER_BUTTON_A */
@@ -49,7 +54,9 @@ static const struct { int x; int y; } button_positions[] = {
     {330, 135},  /* SDL_CONTROLLER_BUTTON_PADDLE2 */
     {132, 175},  /* SDL_CONTROLLER_BUTTON_PADDLE3 */
     {330, 175},  /* SDL_CONTROLLER_BUTTON_PADDLE4 */
+    {0, 0},      /* SDL_CONTROLLER_BUTTON_TOUCHPAD */
 };
+SDL_COMPILE_TIME_ASSERT(button_positions, SDL_arraysize(button_positions) == SDL_CONTROLLER_BUTTON_MAX);
 
 /* This is indexed by SDL_GameControllerAxis. */
 static const struct { int x; int y; double angle; } axis_positions[] = {
@@ -60,6 +67,18 @@ static const struct { int x; int y; double angle; } axis_positions[] = {
     {91,  -20,   0.0},  /* TRIGGERLEFT */
     {375, -20,   0.0},  /* TRIGGERRIGHT */
 };
+SDL_COMPILE_TIME_ASSERT(axis_positions, SDL_arraysize(axis_positions) == SDL_CONTROLLER_AXIS_MAX);
+
+/* This is indexed by SDL_JoystickPowerLevel + 1. */
+static const char* power_level_strings[] = {
+    "unknown", /* SDL_JOYSTICK_POWER_UNKNOWN */
+    "empty",   /* SDL_JOYSTICK_POWER_EMPTY */
+    "low",     /* SDL_JOYSTICK_POWER_LOW */
+    "medium",  /* SDL_JOYSTICK_POWER_MEDIUM */
+    "full",    /* SDL_JOYSTICK_POWER_FULL */
+    "wired",   /* SDL_JOYSTICK_POWER_WIRED */
+};
+SDL_COMPILE_TIME_ASSERT(power_level_strings, SDL_arraysize(power_level_strings) == SDL_JOYSTICK_POWER_MAX + 1);
 
 static SDL_Window *window = NULL;
 static SDL_Renderer *screen = NULL;
@@ -71,6 +90,11 @@ static SDL_Texture *background_front, *background_back, *button, *axis;
 static SDL_GameController *gamecontroller;
 static SDL_GameController **gamecontrollers;
 static int num_controllers = 0;
+static SDL_Joystick *virtual_joystick = NULL;
+static SDL_GameControllerAxis virtual_axis_active = SDL_CONTROLLER_AXIS_INVALID;
+static int virtual_axis_start_x;
+static int virtual_axis_start_y;
+static SDL_GameControllerButton virtual_button_active = SDL_CONTROLLER_BUTTON_INVALID;
 
 static void UpdateWindowTitle()
 {
@@ -120,6 +144,7 @@ static void AddController(int device_index, SDL_bool verbose)
     SDL_JoystickID controller_id = SDL_JoystickGetDeviceInstanceID(device_index);
     SDL_GameController *controller;
     SDL_GameController **controllers;
+    Uint16 firmware_version;
 
     controller_id = SDL_JoystickGetDeviceInstanceID(device_index);
     if (controller_id < 0) {
@@ -151,7 +176,15 @@ static void AddController(int device_index, SDL_bool verbose)
 
     if (verbose) {
         const char *name = SDL_GameControllerName(gamecontroller);
-        SDL_Log("Opened game controller %s\n", name);
+        const char *path = SDL_GameControllerPath(gamecontroller);
+        SDL_Log("Opened game controller %s%s%s\n", name, path ? ", " : "", path ? path : "");
+    }
+
+    firmware_version = SDL_GameControllerGetFirmwareVersion(gamecontroller);
+    if (firmware_version) {
+        if (verbose) {
+            SDL_Log("Firmware version: 0x%x (%d)\n", firmware_version, firmware_version);
+        }
     }
 
     if (SDL_GameControllerHasSensor(gamecontroller, SDL_SENSOR_ACCEL)) {
@@ -216,34 +249,6 @@ static void DelController(SDL_JoystickID controller)
     UpdateWindowTitle();
 }
 
-static SDL_Texture *
-LoadTexture(SDL_Renderer *renderer, const char *file, SDL_bool transparent)
-{
-    SDL_Surface *temp = NULL;
-    SDL_Texture *texture = NULL;
-
-    temp = SDL_LoadBMP(file);
-    if (temp == NULL) {
-        SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s", file, SDL_GetError());
-    } else {
-        /* Set transparent pixel as the pixel at (0,0) */
-        if (transparent) {
-            if (temp->format->BytesPerPixel == 1) {
-                SDL_SetColorKey(temp, SDL_TRUE, *(Uint8 *)temp->pixels);
-            }
-        }
-
-        texture = SDL_CreateTextureFromSurface(renderer, temp);
-        if (!texture) {
-            SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't create texture: %s\n", SDL_GetError());
-        }
-    }
-    if (temp) {
-        SDL_FreeSurface(temp);
-    }
-    return texture;
-}
-
 static Uint16 ConvertAxisToRumble(Sint16 axisval)
 {
     /* Only start rumbling if the axis is past the halfway point */
@@ -306,12 +311,223 @@ static void CyclePS5TriggerEffect()
     SDL_GameControllerSendEffect(gamecontroller, &state, sizeof(state));
 }
 
+static SDL_bool ShowingFront()
+{
+    SDL_bool showing_front = SDL_TRUE;
+    int i;
+
+    if (gamecontroller) {
+        /* Show the back of the controller if the paddles are being held */
+        for (i = SDL_CONTROLLER_BUTTON_PADDLE1; i <= SDL_CONTROLLER_BUTTON_PADDLE4; ++i) {
+            if (SDL_GameControllerGetButton(gamecontroller, (SDL_GameControllerButton)i) == SDL_PRESSED) {
+                showing_front = SDL_FALSE;
+                break;
+            }
+        }
+    }
+    if ((SDL_GetModState() & KMOD_SHIFT) != 0) {
+        showing_front = SDL_FALSE;
+    }
+    return showing_front;
+}
+
+static void SDLCALL VirtualControllerSetPlayerIndex(void *userdata, int player_index)
+{
+    SDL_Log("Virtual Controller: player index set to %d\n", player_index);
+}
+
+static int SDLCALL VirtualControllerRumble(void *userdata, Uint16 low_frequency_rumble, Uint16 high_frequency_rumble)
+{
+    SDL_Log("Virtual Controller: rumble set to %d/%d\n", low_frequency_rumble, high_frequency_rumble);
+    return 0;
+}
+
+static int SDLCALL VirtualControllerRumbleTriggers(void *userdata, Uint16 left_rumble, Uint16 right_rumble)
+{
+    SDL_Log("Virtual Controller: trigger rumble set to %d/%d\n", left_rumble, right_rumble);
+    return 0;
+}
+
+static int SDLCALL VirtualControllerSetLED(void *userdata, Uint8 red, Uint8 green, Uint8 blue)
+{
+    SDL_Log("Virtual Controller: LED set to RGB %d,%d,%d\n", red, green, blue);
+    return 0;
+}
+
+static void OpenVirtualController()
+{
+    SDL_VirtualJoystickDesc desc;
+    int virtual_index;
+
+    SDL_zero(desc);
+    desc.version = SDL_VIRTUAL_JOYSTICK_DESC_VERSION;
+    desc.type = SDL_JOYSTICK_TYPE_GAMECONTROLLER;
+    desc.naxes = SDL_CONTROLLER_AXIS_MAX;
+    desc.nbuttons = SDL_CONTROLLER_BUTTON_MAX;
+    desc.SetPlayerIndex = VirtualControllerSetPlayerIndex;
+    desc.Rumble = VirtualControllerRumble;
+    desc.RumbleTriggers = VirtualControllerRumbleTriggers;
+    desc.SetLED = VirtualControllerSetLED;
+
+    virtual_index = SDL_JoystickAttachVirtualEx(&desc);
+    if (virtual_index < 0) {
+        SDL_Log("Couldn't open virtual device: %s\n", SDL_GetError());
+    } else {
+        virtual_joystick = SDL_JoystickOpen(virtual_index);
+        if (!virtual_joystick) {
+            SDL_Log("Couldn't open virtual device: %s\n", SDL_GetError());
+        }
+    }
+}
+
+static void CloseVirtualController()
+{
+    int i;
+
+    for (i = SDL_NumJoysticks(); i--; ) {
+        if (SDL_JoystickIsVirtual(i)) {
+            SDL_JoystickDetachVirtual(i);
+        }
+    }
+
+    if (virtual_joystick) {
+        SDL_JoystickClose(virtual_joystick);
+        virtual_joystick = NULL;
+    }
+}
+
+static SDL_GameControllerButton FindButtonAtPosition(int x, int y)
+{
+    SDL_Point point;
+    int i;
+    SDL_bool showing_front = ShowingFront();
+
+    point.x = x;
+    point.y = y;
+    for (i = 0; i < SDL_CONTROLLER_BUTTON_TOUCHPAD; ++i) {
+        SDL_bool on_front = (i < SDL_CONTROLLER_BUTTON_PADDLE1 || i > SDL_CONTROLLER_BUTTON_PADDLE4);
+        if (on_front == showing_front) {
+            SDL_Rect rect;
+            rect.x = button_positions[i].x;
+            rect.y = button_positions[i].y;
+            rect.w = BUTTON_SIZE;
+            rect.h = BUTTON_SIZE;
+            if (SDL_PointInRect(&point, &rect)) {
+                return (SDL_GameControllerButton)i;
+            }
+        }
+    }
+    return SDL_CONTROLLER_BUTTON_INVALID;
+}
+
+static SDL_GameControllerAxis FindAxisAtPosition(int x, int y)
+{
+    SDL_Point point;
+    int i;
+    SDL_bool showing_front = ShowingFront();
+
+    point.x = x;
+    point.y = y;
+    for (i = 0; i < SDL_CONTROLLER_AXIS_MAX; ++i) {
+        if (showing_front) {
+            SDL_Rect rect;
+            rect.x = axis_positions[i].x;
+            rect.y = axis_positions[i].y;
+            rect.w = AXIS_SIZE;
+            rect.h = AXIS_SIZE;
+            if (SDL_PointInRect(&point, &rect)) {
+                return (SDL_GameControllerAxis)i;
+            }
+        }
+    }
+    return SDL_CONTROLLER_AXIS_INVALID;
+}
+
+static void VirtualControllerMouseMotion(int x, int y)
+{
+    if (virtual_button_active != SDL_CONTROLLER_BUTTON_INVALID) {
+        if (virtual_axis_active != SDL_CONTROLLER_AXIS_INVALID) {
+            const int MOVING_DISTANCE = 2;
+            if (SDL_abs(x - virtual_axis_start_x) >= MOVING_DISTANCE ||
+                SDL_abs(y - virtual_axis_start_y) >= MOVING_DISTANCE) {
+                SDL_JoystickSetVirtualButton(virtual_joystick, virtual_button_active, SDL_RELEASED);
+                virtual_button_active = SDL_CONTROLLER_BUTTON_INVALID;
+            }
+        }
+    }
+
+    if (virtual_axis_active != SDL_CONTROLLER_AXIS_INVALID) {
+        if (virtual_axis_active == SDL_CONTROLLER_AXIS_TRIGGERLEFT ||
+            virtual_axis_active == SDL_CONTROLLER_AXIS_TRIGGERRIGHT) {
+            int range = (SDL_JOYSTICK_AXIS_MAX - SDL_JOYSTICK_AXIS_MIN);
+            float distance = SDL_clamp(((float)y - virtual_axis_start_y) / AXIS_SIZE, 0.0f, 1.0f);
+            Sint16 value = (Sint16)(SDL_JOYSTICK_AXIS_MIN + (distance * range));
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active, value);
+        } else {
+            float distanceX = SDL_clamp(((float)x - virtual_axis_start_x) / AXIS_SIZE, -1.0f, 1.0f);
+            float distanceY = SDL_clamp(((float)y - virtual_axis_start_y) / AXIS_SIZE, -1.0f, 1.0f);
+            Sint16 valueX, valueY;
+
+            if (distanceX >= 0) {
+                valueX = (Sint16)(distanceX * SDL_JOYSTICK_AXIS_MAX);
+            } else {
+                valueX = (Sint16)(distanceX * -SDL_JOYSTICK_AXIS_MIN);
+            }
+            if (distanceY >= 0) {
+                valueY = (Sint16)(distanceY * SDL_JOYSTICK_AXIS_MAX);
+            } else {
+                valueY = (Sint16)(distanceY * -SDL_JOYSTICK_AXIS_MIN);
+            }
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active, valueX);
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active+1, valueY);
+        }
+    }
+}
+
+static void VirtualControllerMouseDown(int x, int y)
+{
+    SDL_GameControllerButton button;
+    SDL_GameControllerAxis axis;
+
+    button = FindButtonAtPosition(x, y);
+    if (button != SDL_CONTROLLER_BUTTON_INVALID) {
+        virtual_button_active = button;
+        SDL_JoystickSetVirtualButton(virtual_joystick, virtual_button_active, SDL_PRESSED);
+    }
+
+    axis = FindAxisAtPosition(x, y);
+    if (axis != SDL_CONTROLLER_AXIS_INVALID) {
+        virtual_axis_active = axis;
+        virtual_axis_start_x = x;
+        virtual_axis_start_y = y;
+    }
+}
+
+static void VirtualControllerMouseUp(int x, int y)
+{
+    if (virtual_button_active != SDL_CONTROLLER_BUTTON_INVALID) {
+        SDL_JoystickSetVirtualButton(virtual_joystick, virtual_button_active, SDL_RELEASED);
+        virtual_button_active = SDL_CONTROLLER_BUTTON_INVALID;
+    }
+
+    if (virtual_axis_active != SDL_CONTROLLER_AXIS_INVALID) {
+        if (virtual_axis_active == SDL_CONTROLLER_AXIS_TRIGGERLEFT ||
+            virtual_axis_active == SDL_CONTROLLER_AXIS_TRIGGERRIGHT) {
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active, SDL_JOYSTICK_AXIS_MIN);
+        } else {
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active, 0);
+            SDL_JoystickSetVirtualAxis(virtual_joystick, virtual_axis_active+1, 0);
+        }
+        virtual_axis_active = SDL_CONTROLLER_AXIS_INVALID;
+    }
+}
+
 void
 loop(void *arg)
 {
     SDL_Event event;
     int i;
-    SDL_bool showing_front = SDL_TRUE;
+    SDL_bool showing_front;
 
     /* Update to get the current event state */
     SDL_PumpEvents();
@@ -382,6 +598,28 @@ loop(void *arg)
             }
             break;
 
+        case SDL_JOYBATTERYUPDATED:
+            SDL_Log("Controller %d battery state changed to %s\n", event.jbattery.which, power_level_strings[event.jbattery.level + 1]);
+            break;
+
+        case SDL_MOUSEBUTTONDOWN:
+            if (virtual_joystick) {
+                VirtualControllerMouseDown(event.button.x, event.button.y);
+            }
+            break;
+
+        case SDL_MOUSEBUTTONUP:
+            if (virtual_joystick) {
+                VirtualControllerMouseUp(event.button.x, event.button.y);
+            }
+            break;
+
+        case SDL_MOUSEMOTION:
+            if (virtual_joystick) {
+                VirtualControllerMouseMotion(event.motion.x, event.motion.y);
+            }
+            break;
+
         case SDL_KEYDOWN:
             if (event.key.keysym.sym >= SDLK_0 && event.key.keysym.sym <= SDLK_9) {
                 if (gamecontroller) {
@@ -391,6 +629,14 @@ loop(void *arg)
                 }
                 break;
             }
+            if (event.key.keysym.sym == SDLK_a) {
+                OpenVirtualController();
+                break;
+            }
+            if (event.key.keysym.sym == SDLK_d) {
+                CloseVirtualController();
+                break;
+            }
             if (event.key.keysym.sym != SDLK_ESCAPE) {
                 break;
             }
@@ -403,15 +649,7 @@ loop(void *arg)
         }
     }
 
-    if (gamecontroller) {
-        /* Show the back of the controller if the paddles are being held */
-        for (i = SDL_CONTROLLER_BUTTON_PADDLE1; i <= SDL_CONTROLLER_BUTTON_PADDLE4; ++i) {
-            if (SDL_GameControllerGetButton(gamecontroller, (SDL_GameControllerButton)i) == SDL_PRESSED) {
-                showing_front = SDL_FALSE;
-                break;
-            }
-        }
-    }
+    showing_front = ShowingFront();
 
     /* blank screen, set up for drawing this frame. */
     SDL_SetRenderDrawColor(screen, 0xFF, 0xFF, 0xFF, SDL_ALPHA_OPAQUE);
@@ -427,8 +665,8 @@ loop(void *arg)
                     SDL_Rect dst;
                     dst.x = button_positions[i].x;
                     dst.y = button_positions[i].y;
-                    dst.w = 50;
-                    dst.h = 50;
+                    dst.w = BUTTON_SIZE;
+                    dst.h = BUTTON_SIZE;
                     SDL_RenderCopyEx(screen, button, NULL, &dst, 0, NULL, SDL_FLIP_NONE);
                 }
             }
@@ -443,16 +681,16 @@ loop(void *arg)
                     SDL_Rect dst;
                     dst.x = axis_positions[i].x;
                     dst.y = axis_positions[i].y;
-                    dst.w = 50;
-                    dst.h = 50;
+                    dst.w = AXIS_SIZE;
+                    dst.h = AXIS_SIZE;
                     SDL_RenderCopyEx(screen, axis, NULL, &dst, angle, NULL, SDL_FLIP_NONE);
                 } else if (value > deadzone) {
                     const double angle = axis_positions[i].angle + 180.0;
                     SDL_Rect dst;
                     dst.x = axis_positions[i].x;
                     dst.y = axis_positions[i].y;
-                    dst.w = 50;
-                    dst.h = 50;
+                    dst.w = AXIS_SIZE;
+                    dst.h = AXIS_SIZE;
                     SDL_RenderCopyEx(screen, axis, NULL, &dst, angle, NULL, SDL_FLIP_NONE);
                 }
             }
@@ -507,7 +745,7 @@ loop(void *arg)
             }
         }
     }
-
+    SDL_Delay(16);
     SDL_RenderPresent(screen);
 
 #ifdef __EMSCRIPTEN__
@@ -526,7 +764,6 @@ main(int argc, char *argv[])
     char guid[64];
 
     SDL_SetHint(SDL_HINT_ACCELEROMETER_AS_JOYSTICK, "0");
-    SDL_SetHint(SDL_HINT_JOYSTICK_HIDAPI_JOY_CONS, "1");
     SDL_SetHint(SDL_HINT_JOYSTICK_HIDAPI_PS4_RUMBLE, "1");
     SDL_SetHint(SDL_HINT_JOYSTICK_HIDAPI_PS5_RUMBLE, "1");
     SDL_SetHint(SDL_HINT_JOYSTICK_ROG_CHAKRAM, "1");
@@ -560,6 +797,7 @@ main(int argc, char *argv[])
     /* Print information about the controller */
     for (i = 0; i < SDL_NumJoysticks(); ++i) {
         const char *name;
+        const char *path;
         const char *description;
 
         SDL_JoystickGetGUIDString(SDL_JoystickGetDeviceGUID(i),
@@ -568,6 +806,7 @@ main(int argc, char *argv[])
         if (SDL_IsGameController(i)) {
             controller_count++;
             name = SDL_GameControllerNameForIndex(i);
+            path = SDL_GameControllerPathForIndex(i);
             switch (SDL_GameControllerTypeForIndex(i)) {
             case SDL_CONTROLLER_TYPE_AMAZON_LUNA:
                 description = "Amazon Luna Controller";
@@ -575,6 +814,11 @@ main(int argc, char *argv[])
             case SDL_CONTROLLER_TYPE_GOOGLE_STADIA:
                 description = "Google Stadia Controller";
                 break;
+            case SDL_CONTROLLER_TYPE_NINTENDO_SWITCH_JOYCON_LEFT:
+            case SDL_CONTROLLER_TYPE_NINTENDO_SWITCH_JOYCON_RIGHT:
+            case SDL_CONTROLLER_TYPE_NINTENDO_SWITCH_JOYCON_PAIR:
+                description = "Nintendo Switch Joy-Con";
+                break;
             case SDL_CONTROLLER_TYPE_NINTENDO_SWITCH_PRO:
                 description = "Nintendo Switch Pro Controller";
                 break;
@@ -603,10 +847,11 @@ main(int argc, char *argv[])
             AddController(i, SDL_FALSE);
         } else {
             name = SDL_JoystickNameForIndex(i);
+            path = SDL_JoystickPathForIndex(i);
             description = "Joystick";
         }
-        SDL_Log("%s %d: %s (guid %s, VID 0x%.4x, PID 0x%.4x, player index = %d)\n",
-            description, i, name ? name : "Unknown", guid,
+        SDL_Log("%s %d: %s%s%s (guid %s, VID 0x%.4x, PID 0x%.4x, player index = %d)\n",
+            description, i, name ? name : "Unknown", path ? ", " : "", path ? path : "", guid,
             SDL_JoystickGetDeviceVendor(i), SDL_JoystickGetDeviceProduct(i), SDL_JoystickGetDevicePlayerIndex(i));
     }
     SDL_Log("There are %d game controller(s) attached (%d joystick(s))\n", controller_count, SDL_NumJoysticks());
@@ -634,10 +879,10 @@ main(int argc, char *argv[])
     /* scale for platforms that don't give you the window size you asked for. */
     SDL_RenderSetLogicalSize(screen, SCREEN_WIDTH, SCREEN_HEIGHT);
 
-    background_front = LoadTexture(screen, "controllermap.bmp", SDL_FALSE);
-    background_back = LoadTexture(screen, "controllermap_back.bmp", SDL_FALSE);
-    button = LoadTexture(screen, "button.bmp", SDL_TRUE);
-    axis = LoadTexture(screen, "axis.bmp", SDL_TRUE);
+    background_front = LoadTexture(screen, "controllermap.bmp", SDL_FALSE, NULL, NULL);
+    background_back = LoadTexture(screen, "controllermap_back.bmp", SDL_FALSE, NULL, NULL);
+    button = LoadTexture(screen, "button.bmp", SDL_TRUE, NULL, NULL);
+    axis = LoadTexture(screen, "axis.bmp", SDL_TRUE, NULL, NULL);
 
     if (!background_front || !background_back || !button || !axis) {
         SDL_DestroyRenderer(screen);
@@ -650,8 +895,14 @@ main(int argc, char *argv[])
     /* !!! FIXME: */
     /*SDL_RenderSetLogicalSize(screen, background->w, background->h);*/
 
-    if (argv[1] && *argv[1] != '-') {
-        controller_index = SDL_atoi(argv[1]);
+    for (i = 1; i < argc; ++i) {
+        if (SDL_strcmp(argv[i], "--virtual") == 0) {
+            OpenVirtualController();
+        }
+        if (argv[i] && *argv[i] != '-') {
+            controller_index = SDL_atoi(argv[i]);
+            break;
+        }
     }
     if (controller_index < num_controllers) {
         gamecontroller = gamecontrollers[controller_index];
@@ -675,6 +926,7 @@ main(int argc, char *argv[])
         CyclePS5TriggerEffect();
     }
 
+    CloseVirtualController();
     SDL_DestroyRenderer(screen);
     SDL_DestroyWindow(window);
     SDL_QuitSubSystem(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK | SDL_INIT_GAMECONTROLLER);
